@@ -154,6 +154,12 @@ class Color:
     
     def darker(self, level):
         return Color.from_hsl(self.h, self.s, max(self.l - level, 0), self.a)
+    
+    def invert(self):
+        newR = 1.0 - self.r
+        newG = 1.0 - self.g
+        newB = 1.0 - self.b
+        return Color.from_rgb(newR, newG, newB, self.a)
 
     def from_hsl(h, s, l, a=1):
         r, g, b = hsl_to_rgb(h, s, l)
@@ -216,11 +222,20 @@ def find_random(v):
 
 
 def normalize_color(v):
+    if v == -1:
+        return Color.from_rgb(0,0,0,0)
     if v is None:
         return Color.from_rgb(0,0,0,0)
     elif isinstance(v, Color):
         return v
-    elif isinstance(v, Gradient):
+    
+    try:
+        if isinstance(v[0], Color):
+            return v[0]
+    except:
+        pass
+    
+    if isinstance(v, Gradient):
         return v
     elif isinstance(v, float) or isinstance(v, int):
         return Color.from_rgb(v, v, v)
@@ -235,6 +250,8 @@ def normalize_color(v):
             return Color.from_html(v)
     else:
         if len(v) == 1:
+            if v[0] == -1:
+                return Color.from_rgb(0,0,0,0)
             if v[0] == "random":
                 return Color.from_rgb(random(), random(), random(), 1)
             if v[0] == None:
