@@ -1,4 +1,4 @@
-import math, re
+import math, re, inspect
 
 from coldtype.geometry.geometrical import Geometrical
 from coldtype.geometry.point import Point
@@ -36,9 +36,23 @@ for key, (w, h) in list(COMMON_PAPER_SIZES.items()):
     COMMON_PAPER_SIZES["%s-landscape" % key] = (h, w)
 
 
-def align(b, rect, x=Edge.CenterX, y=Edge.CenterY):
-    x = txt_to_edge(x)
-    y = txt_to_edge(y)
+def align(b, rect, x=Edge.CenterX, y=Edge.CenterY, round_result=False):
+    if x in ["NE", "SE", "SW", "NW", "N", "S"]:
+        if x == "NE":
+            x, y = Edge.MaxX, Edge.MaxY
+        elif x == "NW":
+            x, y = Edge.MinX, Edge.MaxY
+        elif x == "SW":
+            x, y = Edge.MinX, Edge.MinY
+        elif x == "SE":
+            x, y = Edge.MaxX, Edge.MinY
+        elif x == "N":
+            x, y = Edge.CenterX, Edge.MaxY
+        elif x == "S":
+            x, y = Edge.CenterX, Edge.MinY
+    else:
+        x = txt_to_edge(x)
+        y = txt_to_edge(y)
     
     xoff = 0
     if x != None:
@@ -59,7 +73,29 @@ def align(b, rect, x=Edge.CenterX, y=Edge.CenterY):
             yoff = -(b.y-rect.y)
     
     #diff = rect.w - b.w
-    return (xoff, yoff)
+    if round_result:
+        return round(xoff), round(yoff)
+    else:
+        return (xoff, yoff)
+
+
+class GeoIterable():
+    def __init__(self, *items):
+        self.items = items
+
+    def __getitem__(self, key):
+        return self.items[key]
+    
+    def map(self, fn):
+        out = []
+        for idx, item in enumerate(self.items):
+            arg_count = len(inspect.signature(fn).parameters)
+            if arg_count == 1:
+                result = fn(item)
+            else:
+                result = fn(idx, item)
+            out.append(result)
+        return GeoIterable(*out)
 
 
 class Rect(Geometrical):
@@ -68,33 +104,40 @@ class Rect(Geometrical):
     
     Constructor handles multiple formats, including:
     
-    * ``x, y, w, h``
-    * ``[x, y, w, h]``
-    * ``w, h`` (x and y default to 0, 0)
+    * `x, y, w, h`
+    * `[x, y, w, h]`
+    * `w, h` (x and y default to 0, 0)
 
-    ``Rect`` objects can be splat'd where lists are expected as individual arguments (as in drawBot), i.e. ``rect(*my_rect)``, or can be passed directly to functions expected a list representation of a rectangle.
+    `Rect` objects can be splat'd where lists are expected as individual arguments (as in drawBot), i.e. `rect(*my_rect)`, or can be passed directly to functions expected a list representation of a rectangle.
     """
 
-    def FromCenter(center, w, h=None):
+    def FromCenter(center, w, h=None) -> "Rect":
         """Create a rect given a center point and a width and height (optional, height will default to width if not specified")"""
         x, y = center
         if not h:
             h = w
         return Rect((x - w/2, y - h/2, w, h))
     
-    def Inches(w, h, dpi=72.0):
+    def Inches(w, h, dpi=72.0) -> "Rect":
         return Rect(w*dpi, h*dpi)
 
     def __init__(self, *rect):
+        if hasattr(rect[0], "rect") and not isinstance(rect[0], Rect):
+            rect = [rect[0].rect]
+
         if isinstance(rect[0], str):
             x, y = 0, 0
             w, h = COMMON_PAPER_SIZES[rect[0].lower()]
         elif isinstance(rect[0], int) or isinstance(rect[0], float):
-            try:
-                x, y, w, h = rect
-            except:
-                w, h = rect
+            if len(rect) == 1:
                 x, y = 0, 0
+                w, h = rect[0], rect[0]
+            else:
+                try:
+                    x, y, w, h = rect
+                except:
+                    w, h = rect
+                    x, y = 0, 0
         else:
             try:
                 x, y, w, h = rect[0]
@@ -106,11 +149,11 @@ class Rect(Geometrical):
         self.w = w
         self.h = h
     
-    def origin(self):
-        """``(x, y)`` as tuple"""
+    def origin(self) -> tuple:
+        """`(x, y)` as tuple"""
         return self.x, self.y
 
-    def from_obj(obj, w=None, h=None):
+    def from_obj(obj, w=None, h=None) -> "Rect":
         r = Rect((0, 0, 0, 0))
         try:
             r.x = obj.x
@@ -130,19 +173,19 @@ class Rect(Geometrical):
             r.y -= h/2
         return r
 
-    def FromExtents(extents):
+    def FromExtents(extents) -> "Rect":
         nw, ne, se, sw = extents
         return Rect(sw[0], sw[1], abs(ne[0] - sw[0]), abs(ne[1] - sw[1]))
     
-    def noop(self, *args, **kwargs):
+    def noop(self, *args, **kwargs) -> "Rect":
         return self
 
-    def FromMnMnMxMx(extents):
-        """Create a rectangle from ``xmin, ymin, xmax, ymax``"""
+    def FromMnMnMxMx(extents) -> "Rect":
+        """Create a rectangle from `xmin, ymin, xmax, ymax`"""
         xmin, ymin, xmax, ymax = extents
         return Rect(xmin, ymin, xmax - xmin, ymax - ymin)
 
-    def FromPoints(*points):
+    def FromPoints(*points) -> "Rect":
         xmin, ymin, xmax, ymax = None, None, None, None
         for p in points:
             if xmin is None or p[0] < xmin:
@@ -155,7 +198,7 @@ class Rect(Geometrical):
                 ymax = p[1]
         return Rect.FromMnMnMxMx([xmin, ymin, xmax, ymax])
 
-    def mnmnmxmx(self):
+    def mnmnmxmx(self) -> tuple:
         """Return extents of rectangle as list"""
         return (self.x, self.y, self.x + self.w, self.y + self.h)
 
@@ -163,7 +206,7 @@ class Rect(Geometrical):
         return self.rect()[key]
 
     def __repr__(self):
-        return "Rect(" + str(self.rect()).replace(" ", "") + ")"
+        return "Rect({:.2f},{:.2f},{:.2f},{:.2f})".format(*self.rect())
     
     def __eq__(self, r):
         try:
@@ -173,54 +216,56 @@ class Rect(Geometrical):
     
     __hash__ = object.__hash__
 
-    def rect(self):
+    def rect(self) -> list:
         """x,y,w,h in list"""
         return [self.x, self.y, self.w, self.h]
     
-    def round(self):
+    xywh = rect
+    
+    def round(self) -> "Rect":
         """round the values in the rectangle to the nearest integer"""
         return Rect([int(round(n)) for n in self])
 
-    def xy(self):
+    def xy(self) -> list:
         """equivalent to origin"""
         return [self.x, self.y]
 
-    def wh(self):
+    def wh(self) -> list:
         """the width and height as a tuple"""
         return [self.w, self.h]
     
     @property
-    def mnx(self):
+    def mnx(self) -> int:
         return self.x
     
     @property
-    def mny(self):
+    def mny(self) -> int:
         return self.y
 
     @property
-    def mxx(self):
+    def mxx(self) -> int:
         return self.x + self.w
     
     @property
-    def mxy(self):
+    def mxy(self) -> int:
         return self.y + self.h
     
     @property
-    def mdx(self):
+    def mdx(self) -> int:
         return self.point("C").x
     
     @property
-    def mdy(self):
+    def mdy(self) -> int:
         return self.point("C").y
 
-    def square(self):
+    def square(self) -> "Rect":
         """take a square from the center of this rect"""
         return Rect(centered_square(self.rect()))
     
-    def align(self, rect, x=Edge.CenterX, y=Edge.CenterY):
-        return self.offset(*align(self, rect, x, y))
+    def align(self, rect, x=Edge.CenterX, y=Edge.CenterY, round_result=False) -> "Rect":
+        return self.offset(*align(self, rect, x, y, round_result=round_result))
     
-    def ipos(self, pt, defaults=(0.5, 0.5), clamp=True):
+    def ipos(self, pt, defaults=(0.5, 0.5), clamp=True) -> tuple:
         """
         Get scaled 0-1 bounded (optional) value
         from a point in a rectangle
@@ -234,31 +279,31 @@ class Rect(Geometrical):
             sy = min(1, max(0, sy))
         return sx, sy
 
-    def divide(self, amount, edge, forcePixel=False):
+    def divide(self, amount, edge, forcePixel=False) -> list:
         """
         **Dividing**
 
         Derived from the behavior of the classic Cocoa function CGRectDivide, which takes a rectangle and breaks it into two pieces, based on a pixel amount and an edge.
 
-        A quick example: assume you have a rectangle, ``r``, defined as such:
+        A quick example: assume you have a rectangle, `r`, defined as such:
 
-        ``r = Rect(0, 0, 300, 100)``
+        `r = Rect(0, 0, 300, 100)`
         
         If you want to break that into a left-hand rectangle that’s 100 pixels wide and a right-hand rectangle that’s 200 pixels wide, you could either say:
         
-        ``left, right = r.divide(100, "mnx")``
+        `left, right = r.divide(100, "mnx")`
         
         `or you could say`
         
-        ``right, left = r.divide(200, "mxx")``
+        `right, left = r.divide(200, "mxx")`
 
-        where ``mxx`` is the rightmost edge, and ``mnx`` is the leftmost edge.
+        where `mxx` is the rightmost edge, and `mnx` is the leftmost edge.
 
         **Centering**
 
         A special use-case is if you want to break a rectangle into `three` rectangles, based on the center "edge", you can do something like this:
 
-        ``left, center, right = r.divide(200, "mdx")``
+        `left, center, right = r.divide(200, "mdx")`
 
         This will result in three rectangles, always left-to-right, where
         left is 50px wide, then center is 200px wide, then right is also 50px wide — anything not in the center will be evenly distributed between left and right, or top-and-bottom in the case of a Y edge.
@@ -266,14 +311,14 @@ class Rect(Geometrical):
         edge = txt_to_edge(edge)
         if edge == Edge.CenterX or edge == Edge.CenterY:
             a, b, c = divide(self.rect(), amount, edge, forcePixel=forcePixel)
-            return Rect(a), Rect(b), Rect(c)
+            return GeoIterable(Rect(a), Rect(b), Rect(c))
         else:
             a, b = divide(self.rect(), amount, edge, forcePixel=forcePixel)
-            return Rect(a), Rect(b)
+            return GeoIterable(Rect(a), Rect(b))
 
-    def subdivide(self, amount, edge):
+    def subdivide(self, amount, edge) -> list:
         """
-        Like ``divide``, but here you specify the number of equal pieces you want (like columns or rows), and then what edge to start at, i.e.
+        Like `divide`, but here you specify the number of equal pieces you want (like columns or rows), and then what edge to start at, i.e.
         
         .. code:: python
             
@@ -288,13 +333,13 @@ class Rect(Geometrical):
         edge = txt_to_edge(edge)
         return [Rect(x) for x in subdivide(self.rect(), amount, edge)]
     
-    def subdivide_with_leading(self, count, leading, edge, forcePixel=True):
+    def subdivide_with_leading(self, count, leading, edge, forcePixel=True) -> list:
         """
         Same as `subdivide`, but inserts leading between each subdivision
         """
         return self.subdivide_with_leadings(count, [leading]*(count-1), edge, forcePixel)
 
-    def subdivide_with_leadings(self, count, leadings, edge, forcePixel=True):
+    def subdivide_with_leadings(self, count, leadings, edge, forcePixel=True) -> list:
         """
         Same as `subdivide_with_leadings`, but inserts leading between each subdivision, indexing the size of the leading from a list of leadings
         """
@@ -304,13 +349,17 @@ class Rect(Geometrical):
         unit = (full - sum(leadings)) / count
         amounts = [val for pair in zip([unit] * count, leadings) for val in pair][:-1]
         return [Rect(x) for x in subdivide(self.rect(), amounts, edge, forcePixel=forcePixel)][::2]
+    
+    sub = subdivide
+    subl = subdivide_with_leading
+    subls = subdivide_with_leadings
 
-    def transform(self, t):
+    def transform(self, t) -> "Rect":
         pts = ["NW", "NE", "SE", "SW"]
         x1, x2, x3, x4 = [t.transformPoint(self.point(pt)) for pt in pts]
         return Rect.FromExtents([x1, x2, x3, x4])
 
-    def rotate(self, degrees, point=None):
+    def rotate(self, degrees, point=None) -> "Rect":
         if Transform:
             t = Transform()
             if not point:
@@ -322,7 +371,7 @@ class Rect(Geometrical):
         else:
             raise Exception("fontTools not installed")
 
-    def scale(self, s, x_edge=Edge.MinX, y_edge=Edge.MinY):
+    def scale(self, s, x_edge=Edge.MinX, y_edge=Edge.MinY) -> "Rect":
         return Rect(scale(self.rect(), s, x_edge, y_edge))
         #x_edge = txt_to_edge(x_edge)
         #y_edge = txt_to_edge(y_edge)
@@ -330,37 +379,45 @@ class Rect(Geometrical):
         #sy = self.h * s
         #return self.take(sx, x_edge, forcePixel=True).take(sy, y_edge, forcePixel=True)
 
-    def union(self, otherRect):
+    def union(self, otherRect) -> "Rect":
         return Rect.FromMnMnMxMx(unionRect(self.mnmnmxmx(), otherRect.mnmnmxmx()))
     
-    def intersection(self, otherRect):
+    def intersection(self, otherRect) -> "Rect":
         return Rect.FromMnMnMxMx(sectRect(self.mnmnmxmx(), otherRect.mnmnmxmx())[1])
     
     sect = intersection
 
-    def take(self, amount, edge, forcePixel=False):
+    def take(self, amount, edge, forcePixel=False) -> "Rect":
         """
-        Like `divide`, but here it just returns the "first" rect from a divide call, not all the resulting pieces, i.e. you can "take" 200px from the center of a rectangle by doing this ``Rect(0, 0, 300, 100).take(200, "mdx")`` which will result in ``Rect([50, 0, 200, 100])``
+        Like `divide`, but here it just returns the "first" rect from a divide call, not all the resulting pieces, i.e. you can "take" 200px from the center of a rectangle by doing this `Rect(0, 0, 300, 100).take(200, "mdx")` which will result in `Rect([50, 0, 200, 100])`
         """
         edge = txt_to_edge(edge)
-        return Rect(take(self.rect(), amount, edge, forcePixel=forcePixel))
+        if not isinstance(edge, Edge):
+            res = self
+            for e in edge:
+                res = res.take(amount, e, forcePixel=forcePixel)
+            return res
+        else:
+            return Rect(take(self.rect(), amount, edge, forcePixel=forcePixel))
 
-    def takeOpposite(self, amount, edge, forcePixel=False):
+    def takeOpposite(self, amount, edge, forcePixel=False) -> "Rect":
         edge = txt_to_edge(edge)
         return self.divide(amount, edge, forcePixel=forcePixel)[1]
 
-    def subtract(self, amount, edge):
+    def subtract(self, amount, edge) -> "Rect":
         """
-        The opposite of ``take``, this will remove and not return a piece of the given amount from the given edge.
+        The opposite of `take`, this will remove and not return a piece of the given amount from the given edge.
         
         Let's say you have a 100px-wide square and you want to drop 10px from the right-hand side, you would do:
 
-        ``Rect(100, 100).subtract(10, Edge.MaxX)``, which leaves you with ``Rect([0, 0, 90, 100])``
+        `Rect(100, 100).subtract(10, Edge.MaxX)`, which leaves you with `Rect([0, 0, 90, 100])`
         """
         edge = txt_to_edge(edge)
         return Rect(subtract(self.rect(), amount, edge))
+    
+    drop = subtract
 
-    def expand(self, amount, edge):
+    def expand(self, amount, edge) -> "Rect":
         edges = None
         if edge == "NW":
             edges = ["mxy", "mnx"]
@@ -378,7 +435,7 @@ class Rect(Geometrical):
     
     add = expand
 
-    def inset(self, dx, dy=None):
+    def inset(self, dx, dy=None) -> "Rect":
         """
         Creates padding in the amount of dx and dy. Also does expansion with negative values, or both at once
         """
@@ -386,60 +443,69 @@ class Rect(Geometrical):
             dy = dx
         return Rect(inset(self.rect(), dx, dy))
     
-    def inset_x(self, dx):
+    def inset_x(self, dx) -> "Rect":
         return self.inset(dx, 0)
     
-    def inset_y(self, dy):
+    def inset_y(self, dy) -> "Rect":
         return self.inset(0, dy)
 
-    def offset(self, dx, dy=None):
+    def offset(self, dx, dy=None) -> "Rect":
         if dy == None:
             dy = dx
         return Rect(offset(self.rect(), dx, dy))
     
-    def offset_x(self, dx):
+    def offset_x(self, dx) -> "Rect":
         return self.offset(dx, 0)
     
-    def offset_y(self, dy):
+    def offset_y(self, dy) -> "Rect":
         return self.offset(0, dy)
     
     o = offset
 
-    def zero(self):
+    def zero(self) -> "Rect":
+        """disregard origin and set it to (0,0)"""
         return Rect((0, 0, self.w, self.h))
+    
+    def nonzero(self) -> bool:
+        """is this rect not just all zeros?"""
+        return not (self.x == 0 and self.y == 0 and self.w == 0 and self.h == 0)
 
     def __add__(self, another_rect):
         return Rect(add(self, another_rect))
 
-    def grid(self, columns=2, rows=2):
+    def grid(self, columns=2, rows=2) -> list:
         """Construct a grid"""
         xs = [row.subdivide(columns, Edge.MinX) for row in self.subdivide(rows, Edge.MaxY)]
         return [item for sublist in xs for item in sublist]
 
-    def pieces(self, amount, edge):
+    def pieces(self, amount, edge) -> list:
         edge = txt_to_edge(edge)
         return [Rect(x) for x in pieces(self.rect(), amount, edge)]
 
-    def edge(self, edge):
+    def edge(self, edge) -> Line:
         edge = txt_to_edge(edge)
         return Line(*edgepoints(self.rect(), edge))
 
-    def center(self):
+    def center(self) -> Point:
         return Point(centerpoint(self.rect()))
 
-    def flip(self, h):
+    def flip(self, h) -> "Rect":
         return Rect([self.x, h - self.h - self.y, self.w, self.h])
 
-    def cardinals(self):
+    def cardinals(self) -> tuple:
         return self.point("N"), self.point("E"), self.point("S"), self.point("W")
 
-    def intercardinals(self):
+    def intercardinals(self) -> tuple:
         return self.point("NE"), self.point("SE"), self.point("SW"), self.point("NW")
     
-    def aspect(self):
+    def FromIntercardinals(pts) -> "Rect":
+        ne, se, sw, nw = pts
+        return Rect(sw[0], sw[1], abs(ne[0] - sw[0]), abs(ne[1] - sw[1]))
+    
+    def aspect(self) -> float:
         return self.h / self.w
     
-    def fit(self, other):
+    def fit(self, other) -> "Rect":
         sx, sy, sw, sh = self
         ox, oy, ow, oh = other
         if ow > oh:
@@ -451,15 +517,15 @@ class Rect(Geometrical):
         #print(sh, fw, fh, other.aspect())
         return self.take(fh, "mdy")
     
-    def avg(self):
+    def avg(self) -> Point:
         pts = self.cardinals()
         return Point(
             sum([p.x for p in pts])/4,
             sum([p.y for p in pts])/4)
 
-    def point(self, eh, ev=Edge.MinX):
+    def point(self, eh, ev=Edge.MinX) -> Point:
         """
-        Get a ``Point`` at a given compass direction, chosen from
+        Get a `Point` at a given compass direction, chosen from
         
         * C
         * W
@@ -495,54 +561,54 @@ class Rect(Geometrical):
     p = point
 
     @property
-    def pne(self): return self.point("NE")
+    def pne(self) -> Point: return self.point("NE")
 
     @property
-    def pe(self): return self.point("E")
+    def pe(self) -> Point: return self.point("E")
 
     @property
-    def ee(self): return self.edge("mxx")
+    def ee(self) -> Line: return self.edge("mxx")
 
     @property
-    def pse(self): return self.point("SE")
+    def pse(self) -> Point: return self.point("SE")
 
     @property
-    def ps(self): return self.point("S")
+    def ps(self) -> Point: return self.point("S")
 
     @property
-    def es(self): return self.edge("mny")
+    def es(self) -> Line: return self.edge("mny")
 
     @property
-    def psw(self): return self.point("SW")
+    def psw(self) -> Point: return self.point("SW")
 
     @property
-    def pw(self): return self.point("W")
+    def pw(self) -> Point: return self.point("W")
 
     @property
-    def ew(self): return self.edge("mnx")
+    def ew(self) -> Line: return self.edge("mnx")
 
     @property
-    def pnw(self): return self.point("NW")
+    def pnw(self) -> Point: return self.point("NW")
 
     @property
-    def pn(self): return self.point("N")
+    def pn(self) -> Point: return self.point("N")
 
     @property
-    def en(self): return self.edge("mxy")
+    def en(self) -> Line: return self.edge("mxy")
 
     @property
-    def pc(self): return self.point("C")
+    def pc(self) -> Point: return self.point("C")
 
     @property
-    def ecx(self): return self.edge("mdx")
+    def ecx(self) -> Line: return self.edge("mdx")
 
     @property
-    def ecy(self): return self.edge("mdy")
+    def ecy(self) -> Line: return self.edge("mdy")
 
-    def intersects(self, other):
+    def intersects(self, other) -> bool:
         return not (self.point("NE").x < other.point("SW").x or self.point("SW").x > other.point("NE").x or self.point("NE").y < other.point("SW").y or self.point("SW").y > other.point("NE").y)
     
-    def maxima(self, n, edge):
+    def maxima(self, n, edge) -> "Rect":
         e = txt_to_edge(edge)
         if e == Edge.MinX:
             return self.setmnx(n)
@@ -559,7 +625,7 @@ class Rect(Geometrical):
         else:
             raise Exception("HELLO")
     
-    def setmnx(self, x):
+    def setmnx(self, x) -> "Rect":
         mnx, mny, mxx, mxy = self.mnmnmxmx()
         return Rect.FromMnMnMxMx([x, mny, mxx, mxy])
     
@@ -719,6 +785,9 @@ class Rect(Geometrical):
             rs.append(_r)
         return rs
     
-    def to_pen(self):
-        from coldtype.pens.draftingpen import DraftingPen
-        return DraftingPen(self)
\ No newline at end of file
+    def interp(self, v, other) -> "Rect":
+        """Interpolate with another rect"""
+        apts = self.intercardinals()
+        bpts = other.intercardinals()
+        ipts = [p1.interp(v, p2) for p1, p2 in zip(apts, bpts)]
+        return Rect.FromIntercardinals(ipts)
\ No newline at end of file
