@@ -1,42 +1,18 @@
-import skia, struct, tempfile
-
+import skia, struct
 from pathlib import Path
 
-from fontTools.pens.transformPen import TransformPen
-from fontTools.pens.basePen import BasePen
-
-from coldtype.pens.datpen import DATPen
-from coldtype.pens.dattext import DATText
+from coldtype.pens.drawablepen import DrawablePenMixin, Gradient
+from coldtype.pens.skiapathpen import SkiaPathPen
+from coldtype.runon.path import P
 from coldtype.img.datimage import DATImage
 from coldtype.geometry import Rect, Point
-from coldtype.pens.drawablepen import DrawablePenMixin, Gradient
-from coldtype.color import Color
 from coldtype.text.reader import Style
+from coldtype.color import Color
 
-
-class SkiaPathPen(BasePen):
-    def __init__(self, dat, h):
-        super().__init__()
-        self.dat = dat
-        self.path = skia.Path()
-
-        tp = TransformPen(self, (1, 0, 0, -1, 0, h))
-        dat.replay(tp)
-    
-    def _moveTo(self, p):
-        self.path.moveTo(p[0], p[1])
-
-    def _lineTo(self, p):
-        self.path.lineTo(p[0], p[1])
-
-    def _curveToOne(self, p1, p2, p3):
-        self.path.cubicTo(p1[0], p1[1], p2[0], p2[1], p3[0], p3[1])
-
-    def _qCurveToOne(self, p1, p2):
-        self.path.quadTo(p1[0], p1[1], p2[0], p2[1])
-
-    def _closePath(self):
-        self.path.close()
+try:
+    from coldtype.text.colr.skia import SkiaShaders
+except ImportError:
+    pass
 
 
 class SkiaPen(DrawablePenMixin, SkiaPathPen):
@@ -50,6 +26,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
         self.alpha = alpha
 
         all_attrs = list(self.findStyledAttrs(style))
+
         skia_paint_kwargs = dict(AntiAlias=True)
         for attrs, attr in all_attrs:
             method, *args = attr
@@ -65,6 +42,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             for k, v in skia_paint_kwargs.items():
                 if not k.startswith("_"):
                     filtered_paint_kwargs[k] = v
+            #filtered_paint_kwargs["AntiAlias"] = False
             self.paint = skia.Paint(**filtered_paint_kwargs)
             if self.blendmode:
                 self.paint.setBlendMode(self.blendmode)
@@ -81,12 +59,41 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
                 pass
             else:
                 canvas.save()
-                did_draw = self.applyDATAttribute(attrs, attr)
+                
+                if method == "COLR":
+                    did_draw = False
+                    self.colr(args[0], dat)
+                else:
+                    did_draw = self.applyDATAttribute(attrs, attr)
+
                 self.paint.setAlphaf(self.paint.getAlphaf()*self.alpha)
                 if not did_draw:
                     canvas.drawPath(self.path, self.paint)
                 canvas.restore()
-    
+
+    def colr(self, data, pen:P):
+        method, args = data
+        shader_fn = getattr(SkiaShaders, method)
+        if shader_fn:
+            ss = pen.data("substructure").copy()
+            ss.invertYAxis(self.rect.h)
+            sval = ss._val.value
+            
+            if method == "drawPathLinearGradient":
+                args["pt1"] = sval[0][1][0]
+                args["pt2"] = sval[1][1][0]
+            elif method == "drawPathSweepGradient":
+                args["center"] = sval[0][1][0]
+            elif method == "drawPathRadialGradient":
+                args["startCenter"] = sval[0][1][0]
+                args["endCenter"] = sval[1][1][0]
+
+            shader = shader_fn(*args.values())
+            self.paint.setStyle(skia.Paint.kFill_Style)
+            self.paint.setShader(shader)
+        else:
+            raise Exception("No matching SkiaShaders function for " + method)
+
     def fill(self, color):
         self.paint.setStyle(skia.Paint.kFill_Style)
         if color:
@@ -95,12 +102,15 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             elif isinstance(color, Color):
                 self.paint.setColor(color.skia())
     
-    def stroke(self, weight=1, color=None, dash=None):
+    def stroke(self, weight=1, color=None, dash=None, miter=None):
         self.paint.setStyle(skia.Paint.kStroke_Style)
         if dash:
             self.paint.setPathEffect(skia.DashPathEffect.Make(*dash))
         if color and weight > 0:
-            self.paint.setStrokeWidth(weight)
+            self.paint.setStrokeWidth(weight*self.scale)
+            if miter:
+                self.paint.setStrokeMiter(miter)
+            
             if isinstance(color, Gradient):
                 self.gradient(color)
             else:
@@ -174,7 +184,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
                 skia.Rect()
                 sr = skia.Rect(*clip.scale(self.scale, "mnx", "mny").flip(self.rect.h).mnmnmxmx())
                 self.canvas.clipRect(sr)
-            elif isinstance(clip, DATPen):
+            elif isinstance(clip, P):
                 sp = SkiaPathPen(clip, self.rect.h)
                 self.canvas.clipPath(sp.path, doAntiAlias=True)
         self.paint.setColor(skia.ColorBLACK)
@@ -182,7 +192,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
         return
     
     def Composite(pens, rect, save_to, scale=1, context=None, style=None):
-        rect = rect.scale(scale)
+        rect = rect.scale(scale).round()
 
         if context:
             info = skia.ImageInfo.MakeN32Premul(rect.w, rect.h)
@@ -221,6 +231,8 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
         stream.flush()
     
     def CompositeToCanvas(pens, rect, canvas, scale=1, style=None):
+        style_ = style
+
         if scale != 1:
             pens.scale(scale, scale, Point((0, 0)))
         
@@ -234,69 +246,90 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             if not pen.visible:
                 return
             
-            if isinstance(pen, DATText):
-                if not isinstance(pen.style, Style):
-                    pen.style = Style(*pen.style[:-1], **pen.style[-1], load_font=0)
+            if "text" in pen._data:
+                text = pen.data("text")
+                style = pen.data("style")
+                frame = pen.ambit()
+
+                if not isinstance(style, Style):
+                    style = Style(*style[:-1], **style[-1], load_font=0)
                 
-                if isinstance(pen.style.font, str):
-                    font = skia.Typeface(pen.style.font)
+                if isinstance(style.font, str):
+                    font = skia.Typeface(style.font)
                 else:
-                    font = skia.Typeface.MakeFromFile(str(pen.style.font.path))
-                    if len(pen.style.variations) > 0:
+                    font = skia.Typeface.MakeFromFile(str(style.font.path))
+                    if len(style.variations) > 0:
                         fa = skia.FontArguments()
                         # h/t https://github.com/justvanrossum/drawbot-skia/blob/master/src/drawbot_skia/gstate.py
                         to_int = lambda s: struct.unpack(">i", bytes(s, "ascii"))[0]
                         makeCoord = skia.FontArguments.VariationPosition.Coordinate
-                        rawCoords = [makeCoord(to_int(tag), value) for tag, value in pen.style.variations.items()]
+                        rawCoords = [makeCoord(to_int(tag), value) for tag, value in style.variations.items()]
                         coords = skia.FontArguments.VariationPosition.Coordinates(rawCoords)
                         fa.setVariationDesignPosition(skia.FontArguments.VariationPosition(coords))
                         font = font.makeClone(fa)
-                pt = pen._frame.point("SW")
+                pt = frame.point("SW")
                 canvas.drawString(
-                    pen.text,
+                    text,
                     pt.x,
                     rect.h - pt.y,
-                    skia.Font(font, pen.style.fontSize),
-                    skia.Paint(AntiAlias=True, Color=pen.style.fill.skia()))
+                    skia.Font(font, style.fontSize),
+                    skia.Paint(AntiAlias=True, Color=style.fill.skia()))
                 return
             elif isinstance(pen, DATImage):
                 paint = skia.Paint(AntiAlias=True)
-                f = pen._frame
+                f = pen.data("frame")
                 canvas.save()
                 for action, *args in pen.transforms:
                     if action == "rotate":
                         deg, pt = args
                         canvas.rotate(-deg, pt.x, pt.y)
                     #print(action, args)
-                paint.setAlphaf(paint.getAlphaf()*data["alpha"])
+                paint.setAlphaf(paint.getAlphaf()*data["alpha"]*pen.alpha)
+                bm = pen.blendmode()
+                if bm:
+                    paint.setBlendMode(bm.to_skia())
+                
                 canvas.drawImage(pen._img, f.x, f.y, paint)
                 canvas.restore()
-                #pen = DATPen().rect(pen.bounds()).img(pen._img, rect=pen.bounds(), pattern=False)
                 return
             
             if state == 0:
-                SkiaPen(pen, rect, canvas, scale, style=style, alpha=data["alpha"])
+                SkiaPen(pen, rect, canvas, scale, style=style_, alpha=data["alpha"])
         
         pens.walk(draw, visible_only=True)
     
-    def Precompose(pens, rect, fmt=None, context=None, scale=1, disk=False):
+    def Precompose(pens, rect, fmt=None, context=None, scale=1, disk=False, style=None):
+        rect = rect.round()
+
+        if scale < 0:
+            rescale = abs(scale)
+            scale = 1
+        else:
+            rescale = None
+
+        sr = rect
+        if scale != 1:
+            sr = rect.scale(scale).round()
         rect = rect.round()
         if context:
-            info = skia.ImageInfo.MakeN32Premul(rect.w, rect.h)
+            info = skia.ImageInfo.MakeN32Premul(sr.w, sr.h)
             surface = skia.Surface.MakeRenderTarget(context, skia.Budgeted.kNo, info)
             assert surface is not None
         else:
-            #print("CPU PRECOMPOSE")
-            surface = skia.Surface(rect.w, rect.h)
+            surface = skia.Surface(sr.w, sr.h)
         
         with surface as canvas:
-            SkiaPen.CompositeToCanvas(pens.translate(-rect.x, -rect.y), rect, canvas)
+            canvas.save()
+            canvas.scale(scale, scale)
+            SkiaPen.CompositeToCanvas(pens.translate(-rect.x, -rect.y), rect, canvas, style=style)
+            canvas.restore()
         img = surface.makeImageSnapshot()
-        if scale != 1:
-            x, y, w, h = rect.scale(scale)
+        if rescale is not None:
+            x, y, w, h = rect.scale(rescale)
             img = img.resize(int(w), int(h))
         
         if disk:
+            Path(disk).parent.mkdir(exist_ok=True, parents=True)
             img.save(disk, skia.kPNG)
             #return disk
         return img
