@@ -1,3 +1,4 @@
+import pytest
 from coldtype import *
 from functools import partial
 from random import Random
@@ -12,9 +13,9 @@ Style.RegisterShorthandPrefix("≈", "~/Type/fonts/fonts")
 
 
 def add_grid(render, result):
-    return DATPens([
-        DATPens(result),
-        DATPen().gridlines(render.rect).s(0, 0.1).sw(1).f(None)
+    return P([
+        P(result),
+        P().gridlines(render.rect).s(0, 0.1).sw(1).f(None)
     ])
 
 
@@ -23,5 +24,8 @@ def show_error(r, txt):
 
 
 class test(renderable):
-    def __init__(self, rect=(1000, 500), bg=1, postfn=add_grid, **kwargs):
+    def __init__(self, rect=(800, 200), bg=1, postfn=add_grid, **kwargs):
+        if isinstance(rect, int):
+            rect = (800, rect)
+        
         super().__init__(rect=rect, bg=bg, postfn=postfn, **kwargs)
\ No newline at end of file
