@@ -1,7 +1,3 @@
-from coldtype.pens.datpen import DATPen, DATPens
-
-import coldtype.text.composer
-
 from coldtype.geometry.rect import Rect
 from coldtype.text.reader import Style, StyledString, SegmentedString, normalize_font_path, Font, FontNotFoundException
 from coldtype.text.composer import Slug, Lockup, Graf, GrafStyle, T2L, Composer, StSt, Glyphwise
