@@ -1,7 +1,6 @@
 import re
 
-from coldtype.pens.draftingpen import DraftingPen
-from coldtype.pens.draftingpens import DraftingPens
+from coldtype.runon.path import P
 from coldtype.text.composer import Graf, GrafStyle, Lockup
 from coldtype.text.reader import StyledString, Style
 from coldtype.color import hsl
@@ -19,7 +18,7 @@ except ImportError:
     highlight = None
     pass
 
-class RichText(DraftingPens):
+class RichText(P):
     """Very experimental module to support rich-text from annotated strings, like a super-minimal-but-open-ended subset of markdown, inspired by the way rich text is built up in the time.nle.premiere DPS subclass in coldtype"""
     def __init__(self,
         rect,
@@ -32,16 +31,17 @@ class RichText(DraftingPens):
         visible_boundaries=[" "],
         invisible_boundaries=[],
         union_styles=True,
-        blankfill="¶",
+        spacer="¶",
         strip=True,
         strip_lines=False):
 
         super().__init__()
+        
         self.tag_delimiters = tag_delimiters
         self.visible_boundary_chars = visible_boundaries
         self.invisible_boundary_chars = invisible_boundaries
         self.union_styles = union_styles
-        self.blankfill = blankfill
+        self.spacer = spacer
         
         if isinstance(text, PurePath):
             text = text.read_text()
@@ -51,7 +51,7 @@ class RichText(DraftingPens):
         if strip_lines:
             text = "\n".join([l.strip() for l in text.split("\n")])
         
-        self._pens = self.parse_block(text, render_text_fn, rect, fit, graf_style or leading)._pens
+        self._els = self.parse_block(text, render_text_fn, rect, fit, graf_style or leading)._els
 
     def parse_block(self, txt, render_text_fn, rect, fit, graf_style):
         parsed_lines = []
@@ -88,7 +88,7 @@ class RichText(DraftingPens):
                 else:
                     slugs[si] += line[i]
                 i += 1
-            
+
             if not list(slugs.values())[-1]:
                 line_meta = list(metas.values())[-1]
                 last_key = list(slugs.keys())[-1]
@@ -115,8 +115,8 @@ class RichText(DraftingPens):
             #print(">", pl)
             if pl:
                 parsed_lines.append(pl)
-            elif self.blankfill:
-                parsed_lines.append([[self.blankfill, ["blank"]]])
+            elif self.spacer:
+                parsed_lines.append([[self.spacer, ["blank"]]])
 
         lines = []
         groupings = []
@@ -176,31 +176,35 @@ class RichText(DraftingPens):
             if fit:
                 lockup.fit(fit)
             lockups.append(lockup)
-        graf = Graf(lockups, rect, graf_style)
+        
+        graf = Graf(lockups, rect, graf_style, no_frames=True)
         pens = graf.pens()#.align(rect, x="minx")
-        group_pens = DraftingPens() # TODO configurable?
+        pens._frame = None
 
         pens.reversePens()
         for line in pens:
             line.reversePens()
             for slug in line:
                 slug.reversePens()
-        return pens
+        return pens.zero()
     
-    def filter_style(self, style):
-        return self.pfilter(lambda i, p: style in p.data.get("style_names", []))
+    def findStyle(self, style, modfn):
+        return self.find(lambda p: style in p.data("style_names", []), modfn)
     
-    def filter_text(self, text, flags=re.I):
-        return self.pfilter(lambda i, p: re.match(text, p.data.get("txt", ""), flags=flags))
+    def findText(self, text, modfn, flags=re.I):
+        return self.find(lambda p: re.match(text, p.data("txt", ""), flags=flags), modfn)
     
-    def remove_blanklines(self, blank=None):
-        if not blank and self.blankfill:
-            blank = self.blankfill
+    def removeSpacers(self, spacer=None, clean=True):
+        if not spacer and self.spacer:
+            spacer = self.spacer
         
-        for line in self._pens:
-            txt = reduce(lambda acc, p: p.data.get("txt", "") + acc, line, "")
-            if txt == blank:
-                line._pens = [DraftingPen()]
+        for line in self._els:
+            txt = reduce(lambda acc, p: p.data("txt", "") + acc, line, "")
+            if txt == spacer:
+                line._els = []
+        
+        if clean:
+            return self.removeBlanks()
         
         return self
 
